/*
Copyright © 2024 filifa

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/
package cmd

import (
	"log"
	"os"

	"github.com/spf13/cobra"
	"github.com/veandco/go-sdl2/sdl"
)

var preset string

func handleWindowEvent(event *sdl.WindowEvent, m *milkDropWindow) {
	switch event.Event {
	case sdl.WINDOWEVENT_RESIZED:
		w, h := event.Data1, event.Data2
		m.resize(w, h)
	}
}

func handleEvent(event sdl.Event, m *milkDropWindow) bool {
	switch event.(type) {
	case *sdl.QuitEvent:
		return false
	case *sdl.WindowEvent:
		event := event.(*sdl.WindowEvent)
		handleWindowEvent(event, m)
	}

	return true
}

func milkbucket(cmd *cobra.Command, args []string) {
	fi, err := os.Stdin.Stat()
	if err != nil {
		panic(err)
	}
	size := fi.Size()
	if size <= 0 {
		log.Fatal("stdin is empty")
	}

	err = sdl.Init(sdl.INIT_VIDEO)
	if err != nil {
		panic(err)
	}
	defer sdl.Quit()

	m, err := newMilkDropWindow(800, 600)
	if err != nil {
		panic(err)
	}
	defer m.destroy()

	m.loadPreset(preset)

	running := true
	for running {
		for event := sdl.PollEvent(); event != nil; event = sdl.PollEvent() {
			running = handleEvent(event, m)
			if !running {
				break
			}
		}

		if !running {
			break
		}

		running, err = m.render()
		if err != nil {
			panic(err)
		}
	}
}

// rootCmd represents the base command when called without any subcommands
var rootCmd = &cobra.Command{
	Use:   "milkbucket",
	Short: "A brief description of your application",
	Long: `A longer description that spans multiple lines and likely contains
examples and usage of using your application. For example:

Cobra is a CLI library for Go that empowers applications.
This application is a tool to generate the needed files
to quickly create a Cobra application.`,
	Run: milkbucket,
}

// Execute adds all child commands to the root command and sets flags appropriately.
// This is called by main.main(). It only needs to happen once to the rootCmd.
func Execute() {
	err := rootCmd.Execute()
	if err != nil {
		os.Exit(1)
	}
}

func init() {
	// Here you will define your flags and configuration settings.
	// Cobra supports persistent flags, which, if defined here,
	// will be global for your application.

	// Cobra also supports local flags, which will only run
	// when this action is called directly.
	rootCmd.Flags().StringVarP(&preset, "preset", "p", "", "Preset file to use")
}
